import json

from botocore.config import Config
from botocore.exceptions import ClientError
from hca_util.bucket_policy import new_policy_statement
from hca_util.common import gen_uuid
from hca_util.common import print_err


class CmdCreate:
    """
    user: wrangler only
    aws resource or client used in command - s3 client (put_object), s3 resource (BucketPolicy)
    """

    def __init__(self, aws, args):
        self.aws = aws
        self.args = args

    def run(self):

        if self.aws.is_contributor:
            return False, 'You don\'t have permission to use this command'

        area_name = self.args.NAME
        perms = self.args.p  # optional str, default 'ux'

        # generate random uuid prefix for area name
        area_id = gen_uuid()

        try:
            metadata = {'name': area_name, 'perms': perms}

            s3_client = self.aws.common_session.client('s3', endpoint_url='https://uk1s3.embassy.ebi.ac.uk', config=Config(s3={'addressing_style': 'path'}))
            s3_client.put_object(Bucket=self.aws.bucket_name, Key=(area_id + '/'), Metadata=metadata)

            return True, 'Created upload area with UUID ' + area_id + ' and name ' + area_name

        except Exception as e:
            print_err(e, 'create')



from hca_util.common import serialize, deserialize
from hca_util.settings import LOCAL_STATE_FILE


class LocalState:

    def __init__(self):
        self.bucket = None # default
        self.selected_area = None
        self.known_areas = []
        # potential other local state vars
        self.tmp_access_key = None
        self.tmp_secret_key = None
        self.token = None

    def select_area(self, area_name):
        self.selected_area = area_name

        if area_name not in self.known_areas:
            self.known_areas.append(area_name)

    def unselect_area(self):
        self.selected_area = None

    def __str__(self):
        if self.selected_area:
            s = f'Selected {self.selected_area}'
        else:
            s = 'No area selected'

        s += '\nKnown areas:\n'
        known_areas_minus_selected = [d for d in self.known_areas if d != self.selected_area]

        if known_areas_minus_selected:
            s += '\n'.join(map(str, known_areas_minus_selected))
        else:
            s += 'None'

        return s

def get_local_state():
    obj = deserialize(LOCAL_STATE_FILE)
    if obj and isinstance(obj, LocalState):
        return obj
    return LocalState()


def set_local_state(obj):
    serialize(LOCAL_STATE_FILE, obj)


def get_selected_area():
    obj = deserialize(LOCAL_STATE_FILE)
    if obj and isinstance(obj, LocalState):
        return obj.selected_area
    return None


def set_selected_area(area_name):
    obj = deserialize(LOCAL_STATE_FILE)
    if obj is None or not isinstance(obj, LocalState):
        obj = LocalState()
    obj.select_area(area_name)
    serialize(LOCAL_STATE_FILE, obj)


def get_bucket():
    obj = deserialize(LOCAL_STATE_FILE)
    if obj and isinstance(obj, LocalState) and hasattr(obj, 'bucket'):
        return obj.bucket
    return None

    
def set_bucket(bucket):
    obj = deserialize(LOCAL_STATE_FILE)
    if obj is None or not isinstance(obj, LocalState):
        obj = LocalState()
    obj.bucket(bucket)
    serialize(LOCAL_STATE_FILE, obj)
